/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: matchix.c,v 1.1.1.1 2005/03/15 15:57:28 laire Exp $
 
    Desc:
    Lang: English
*/

/*****************************************************************************
 
    NAME */

#include "cxintern.h"
#include <libraries/commodities.h>
#include <devices/inputevent.h>
#include <proto/commodities.h>

#include <aros/debug.h>

//#define DEBUG_MATCHIX(x)	;//if ((event->ie_Class == IECLASS_RAWMOUSE) || (event->ie_Class == IECLASS_POINTERPOS) || (event->ie_Class == IECLASS_NEWPOINTERPOS)) x;
#define DEBUG_MATCHIX(x)	;//if ((event->ie_Class == IECLASS_RAWKEY) || (event->ie_Code == 0x23)) x;


AROS_LH2(BOOL, MatchIX,

         /*  SYNOPSIS */

         AROS_LHA(struct InputEvent *, event, A0),
         AROS_LHA(IX *               , ix   , A1),

         /*  LOCATION */

         struct Library *, CxBase, 34, Commodities)

/*  FUNCTION
 
    Check if an input event matches an input expression.
 
    INPUTS
 
    event  --  the input event to match against the input expression
    ix     --  the input expression
 
    RESULT
 
    TRUE if the input event matches the input expression, FALSE otherwise.
 
    NOTES
 
    Applications don't normally need this function as filter objects take
    care of the event filtering. However, this function is in some cases
    nice to have.
 
    EXAMPLE
 
    BUGS
 
    SEE ALSO
 
    <libraries/commodities.h>, ParseIX()
 
    INTERNALS
 
    HISTORY
 
******************************************************************************/

{
	AROS_LIBFUNC_INIT

	UWORD temp;
	UWORD qual;

	DEBUG_MATCHIX(dprintf("MatchIX: ev[0x%lx, 0x%lx, 0x%lx] ix[0x%lx, (0x%lx, 0x%lx), (0x%lx, 0x%lx, 0x%lx)]\n",
	                      event->ie_Class, event->ie_Code, event->ie_Qualifier,
	                      ix->ix_Class, ix->ix_Code, ix->ix_CodeMask,
	                      ix->ix_Qualifier, ix->ix_QualMask, ix->ix_QualSame));

	if (ix->ix_Class == IECLASS_NULL)
	{
		DEBUG_MATCHIX(dprintf("MatchIX: IECLASS_NULL, match\n"));
		return TRUE;
	}

	if (event->ie_Class != ix->ix_Class)
	{
		DEBUG_MATCHIX(dprintf("MatchIX: event->ie_Class != ix->ix_Class, no match\n"));
		return FALSE;
	}

	DEBUG_MATCHIX(dprintf("Code: ie %lx ix: %lx\n", event->ie_Code, ix->ix_Code));

	if (((ix->ix_Code ^ event->ie_Code) & ix->ix_CodeMask))
	{
		DEBUG_MATCHIX(dprintf("MatchIX: ((ix->ix_Code ^ event->ie_Code) & ix->ix_CodeMask), no match\n"));
		return FALSE;
	}

	temp = event->ie_Qualifier;
	DEBUG_MATCHIX(dprintf("Code: temp %lx\n", temp));

	if ((qual = ix->ix_QualSame))
	{
		if (qual & IXSYM_SHIFT)
		{
			if (temp & IXSYM_SHIFTMASK)
			{
				temp |= IXSYM_SHIFTMASK;
			}
		}

		if (qual & IXSYM_CAPS)
		{
			if (temp & IXSYM_CAPSMASK)
			{
				temp |= IXSYM_CAPSMASK;
			}
		}

		if (qual & IXSYM_ALT)
		{
			if (temp & IXSYM_ALTMASK)
			{
				temp |= IXSYM_ALTMASK;
			}
		}
	}
	DEBUG_MATCHIX(dprintf("Code: temp %lx\n", temp));

	if (((temp ^ ix->ix_Qualifier) & ix->ix_QualMask) == 0)
	{
		DEBUG_MATCHIX(dprintf("MatchIX: match, return true\n"));
		return TRUE;
	}
	else
	{
		DEBUG_MATCHIX(dprintf("MatchIX: no match, return false\n"));
		return FALSE;
	}

	AROS_LIBFUNC_EXIT
} /* MatchIX */
